
// General class overview:  This class is designed to control the reading and writing of the applications configuration file.  Configuration parameters currently
// 	include:
//		TENA.ORBdefaultInitRef		- the parameter to be passed to the TENA::Middleware::Configuration object as the ORBdefaultInitRef attribute.
//		TENA.executionName			- the parameter to be passed to the TENA::Middleware::Configuration object as the executionName attribute.
//		TENA.ORBlistenEndpoints		- the parameter to be passed to the TENA::Middleware::Configuration object as the ORBlistenEndpoints attribute.
//		TENA.sessionName			- the parameter to be passed to the TENA::Middleware::Configuration object as the sessionName attribute.
// 	Methods of this class are NOT threadsafe.

#include "Configuration.h"

#include <iostream>
#include <fstream>

ConfigurationFile::ConfigurationFile(void)
	: _dblLtpLatitude(0)
	, _dblLtpLongitude(0)
	, _dblLtpAzimuth(0)
	, _dblLtpXFalseOrigin(0)
	, _dblLtpYFalseOrigin(0)
	, _dblLtpHeightOffset(0)
{
}

ConfigurationFile::~ConfigurationFile(void)
{
}

std::string ConfigurationFile::GetConfigurationFileName()
{
	return _strFileName;
}

void ConfigurationFile::SetConfigurationFileName(std::string strFileName)
{
	_strFileName = strFileName;
}


void ConfigurationFile::ReadFile()
{
	try
	{
		char str[2000];
		std::fstream inputFile(_strFileName.c_str(), std::ios::in);
		if(!inputFile.is_open())
		{
			std::cout << "No Configuration file (" << _strFileName << ") found." << std::endl;
			return;
		}
		while(!inputFile.eof()) 
        {
			inputFile.getline(str,2000);
			std::string strLine(str), strParameter, sValue;
			if(!strLine.empty())
			{
				strParameter = strLine.substr(0, strLine.find('='));
				sValue = strLine.substr(strLine.find('=') + 1, strLine.length());
				if(strParameter == "TENA.ORBdefaultInitRef")
					_strORBdefaultInitRef = sValue;
				else if(strParameter == "TENA.executionName")
					_strExecutionName = sValue;
				else if(strParameter == "TENA.ORBlistenEndpoints")
					_strORBlistenEndpoints = sValue;
				else if(strParameter == "TENA.sessionName")
					_strSessionName = sValue;
				else if(strParameter == "TENA.LTP_latitude")
					_dblLtpLatitude = atof(sValue.c_str());
				else if(strParameter == "TENA.LTP_longitude")
					_dblLtpLongitude = atof(sValue.c_str());
				else if(strParameter == "TENA.LTP_azimuth")
					_dblLtpAzimuth = atof(sValue.c_str());
				else if(strParameter == "TENA.LTP_XFalseOrigin")
					_dblLtpXFalseOrigin = atof(sValue.c_str());
				else if(strParameter == "TENA.LTP_YFalseOrigin")
					_dblLtpYFalseOrigin = atof(sValue.c_str());
				else if(strParameter == "TENA.LTP_HeightOffset")
					_dblLtpHeightOffset = atof(sValue.c_str());
			}
		}
	}
	catch(std::exception const &ex)
	{
		std::cout << "Exception processing Configuration file (" << _strFileName << "): " << ex.what() << std::endl;
	}
	catch(...)
	{
		std::cout << "Unknown exception processing Configuration file (" << _strFileName << ")." << std::endl;
	}
}

std::string ConfigurationFile::GetORBdefaultInitRef() 
{ 
	return _strORBdefaultInitRef; 
}

std::string ConfigurationFile::GetExecutionName() 
{ 
	return _strExecutionName; 
}

std::string ConfigurationFile::GetSessionName() 
{ 
	return _strSessionName; 
}

std::string ConfigurationFile::GetORBlistenEndpoints()  
{ 
	return _strORBlistenEndpoints; 
}

double ConfigurationFile::GetLocalTangentPlaneLatitude()  
{ 
	return _dblLtpLatitude; 
}

double ConfigurationFile::GetLocalTangentPlaneLongitude()  
{ 
	return _dblLtpLongitude; 
}

double ConfigurationFile::GetLocalTangentPlaneAzimuth()  
{ 
	return _dblLtpAzimuth; 
}

double ConfigurationFile::GetLocalTangentPlaneXFalseOrigin()  
{ 
	return _dblLtpXFalseOrigin; 
}

double ConfigurationFile::GetLocalTangentPlaneYFalseOrigin()  
{ 
	return _dblLtpYFalseOrigin; 
}

double ConfigurationFile::GetLocalTangentPlaneHeightOffset()  
{ 
	return _dblLtpHeightOffset; 
}


