// KbutsliderCtrl.cpp : implementation file
//
//  Found on CodeGURU.com
//    overrides MFC SLider control with round "knob" type control for full 360 degree of input
//    My changes are marked with "// mg" .
//    Mike Greene - Trideum Corp.
//    25 July 2005

#include "stdafx.h"
#include "KspecialControls.h"
#include "KbutsliderCtrl.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern int globalOSGPanValue;  // mg  added extern value to pass to OSG for Pan angle

/////////////////////////////////////////////////////////////////////////////
// KbutsliderCtrl

KbutsliderCtrl::KbutsliderCtrl() 
{
	m_nKnobRadius = 8;
	m_bDragging = false;
	m_nKnobNum = 1;
	m_nKnobCrt = m_nKnobBfr = 0;
	m_nKnobPosi[0] = 0;
	m_nKnobPosi[1] = 90;	
	s_ButStateNew = s_ButStateOld = s_ClikState = 0 ;
}

KbutsliderCtrl::~KbutsliderCtrl()
{
}


void KbutsliderCtrl::operator delete(void* p)
{
	delete(p);
}


BEGIN_MESSAGE_MAP(KbutsliderCtrl, CSliderCtrl)
	//{{AFX_MSG_MAP(KbutsliderCtrl)
	ON_WM_PAINT()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// KbutsliderCtrl message handlers

/* ------------------------------------------------------------------------------------------ */

CPoint KbutsliderCtrl::CalculateKnobPosi(int angle) {
	const int nMin = GetRangeMin();
	const int nMax = GetRangeMax()+1;
	int nPos = (((angle-nMin)*360/(nMax-nMin)) + 360) % 360;
	const double dPos = ((double)(nPos))*pi/180.0;
	const CPoint ptKnobCenter(m_ptCenter.x + m_nRadius * sin(dPos), m_ptCenter.y - m_nRadius * cos(dPos));
	return ptKnobCenter;
}

int KbutsliderCtrl::GetCurentAngle(CPoint mpoz) {
	int nMin, nMax, nNewPos;
	CSize szDelta;
	double dNewPos = 0.0;
	nMin = GetRangeMin();
	nMax = GetRangeMax() + 1;
	szDelta = mpoz - m_ptCenter;
	if(szDelta.cx != 0) {
		dNewPos = 90.0 - atan(-(double)szDelta.cy / (double)szDelta.cx) * 180.0 / pi;
	}
	if(((szDelta.cx == 0) && (szDelta.cy >= 0)) || (szDelta.cx < 0)) {
		dNewPos += 180.0;
	}
	while(dNewPos < 0.0) dNewPos += 360.0;
	while(dNewPos >= 360.0) dNewPos -= 360.0;
	nNewPos = nMin + (int)(dNewPos*(nMax-nMin)/360.0);
	return nNewPos;
}

int KbutsliderCtrl::DetectNearestKnob(int angle) {
	int i,sl;
	for ( i=0,sl=0; i<m_nKnobNum; i++ ) {
		sl = abs(m_nKnobPosi[i] - angle) < abs(m_nKnobPosi[sl] - angle) ? i : sl;
	}
	return sl;
}

/* ------------------------------------------------------------------------------------------ */

void KbutsliderCtrl::DrawCircle(CDC *pDC, CPoint p, LONG lRadius, COLORREF crColour, BOOL bDashed) {
	LONG i, x0,y0,x1,y1;
	//Check to see that the coordinates are valid
	ASSERT( (p.x + lRadius <= LONG_MAX) && (p.y + lRadius <= LONG_MAX) );
	ASSERT( (p.x - lRadius >= LONG_MIN) && (p.y - lRadius >= LONG_MIN) );
	if ( bDashed ) {
		for ( i=0; i<360; i++ ) {
			x0 = lRadius * cos(2*pi*i/360);
			y0 = lRadius * sin(2*pi*i/360);
		//	x1 = lRadius * cos(2*pi*(i+1)/360);
		//	y1 = lRadius * sin(2*pi*(i+1)/360);
			pDC->SetPixelV(p.x + x0, p.y + y0, crColour);		
		}
	}
}

void KbutsliderCtrl::DrawCircle(CDC *pDC, CPoint p, LONG lRadius, COLORREF crBright, COLORREF crDark) {
	LONG i, x0,y0,x1,y1;
	COLORREF crColour;
	BYTE Red, Green, Blue;
	double Weight;
	for ( i=0; i<360; i++ ) {
		Weight = 0.5*(cos(lightdir - ((double)i-180)/360*2*pi)+1.0);
		Red   = (BYTE) (Weight*GetRValue(crBright) + (1.0-Weight)*GetRValue(crDark));
		Green = (BYTE) (Weight*GetGValue(crBright) + (1.0-Weight)*GetGValue(crDark));
		Blue  = (BYTE) (Weight*GetBValue(crBright) + (1.0-Weight)*GetBValue(crDark));
		crColour = RGB(Red, Green, Blue);
		x0 = lRadius * cos(2*pi*(i-180)/360);
		y0 = lRadius * sin(2*pi*(i-180)/360);
		x1 = lRadius * cos(2*pi*(i+1-180)/360);
		y1 = lRadius * sin(2*pi*(i+1-180)/360);
	//	pDC->SetPixelV(p.x + lXoffset, p.y + lYoffset, crColour);				
		DrawLine(pDC,p.x + x0,p.y + y0,p.x + x1,p.y + y1,crColour);
	}
}

void KbutsliderCtrl::DrawText ( CDC *pDC ) {
	CString mesaj;
	mesaj.Format("Pan in Deg: %d", m_nKnobPosi[m_nKnobCrt]);
// mg this line added to extract slider value to extern variable and pass it to OSG for use
	globalOSGPanValue = m_nKnobPosi[m_nKnobCrt];
	//mesaj.Format("Ang: %d -\ Deg: %d", m_nKnobCrt, m_nKnobPosi[m_nKnobCrt]);  //original mg
	CWnd* pOwner = GetParentOwner();
	if ( pOwner ){
		CFont* pFont = pOwner->GetFont();
		pDC->SelectObject(pFont);
	}
	const CSize szExtent = pDC->GetTextExtent(mesaj);
	const CPoint ptText = CPoint(m_ptCenter.x - szExtent.cx/2, m_ptCenter.x - szExtent.cy/2);
	pDC->SetBkMode(TRANSPARENT);
	if ( !IsWindowEnabled() ) {
		pDC->DrawState(ptText, szExtent, mesaj, DSS_DISABLED, TRUE, 0, (HBRUSH)NULL);
	} else {
		pDC->TextOut(ptText.x, ptText.y, mesaj);
	}
}

void KbutsliderCtrl::DrawLine(CDC *pDC, int x1, int y1, int x2, int y2, COLORREF clr) {
	int x,y;
	if ( abs(x2-x1) < abs(y2-y1) ) {
		for ( y=y1; (y2 > y1) ? y<=y2 : y>=y2; y+=((y2 > y1) ? +1 : -1) ) {
			x = (x2!=x1) ?  (x1 - (x1-x2)/(y1-y2)*(y1-y)) : x1;
			pDC->SetPixelV(x,y,clr);
		}
	} else {
		for ( x=x1; (x2 > x1) ? x<=x2 : x>=x2; x+=((x2 > x1) ? +1 : -1) ) {
			y = (y2!=y1) ? (y1 - (y1-y2)/(x1-x2)*(x1-x)) : y1;
			pDC->SetPixelV(x,y,clr);
		}
	}
}

void KbutsliderCtrl::DrawBitmap(CDC *pDC, int state) {	
	CSize rr(40,40);
	CBitmap *pOldBitmap;
	CDC *w = new CDC;
	w->CreateCompatibleDC(pDC);
	pOldBitmap = w->SelectObject(&c_mybitmap);
	pDC->BitBlt(m_ptCenter.x-rr.cx/2,m_ptCenter.y-rr.cy/2,rr.cx,rr.cy,w,rr.cx*state,0,SRCCOPY);	
	w-> SelectObject(pOldBitmap);	
	delete w;
}

/* ------------------------------------------------------------------------------------------ */

void KbutsliderCtrl::OnPaint() {
	int j;
	CPaintDC dc(this); // device context for painting
//	CDC* pDC = &dc;
	CMemDC memDC(&dc);
	CMemDC *pDC = &memDC;


	int nRadius = m_nRadius;
	CRect rc;
	GetClientRect(rc);
	pDC->SelectStockObject(NULL_BRUSH);
	pDC->FillSolidRect(rc, ::GetSysColor(COLOR_BTNFACE));	
	DrawCircle(pDC, m_ptCenter, nRadius--, ::GetSysColor(COLOR_3DDKSHADOW), ::GetSysColor(COLOR_3DHIGHLIGHT));
	DrawCircle(pDC, m_ptCenter, nRadius, ::GetSysColor(COLOR_3DSHADOW), ::GetSysColor(COLOR_3DLIGHT));
	nRadius-=2;
	DrawCircle(pDC, m_ptCenter, nRadius--, ::GetSysColor(COLOR_3DHIGHLIGHT), ::GetSysColor(COLOR_3DDKSHADOW));
	DrawCircle(pDC, m_ptCenter, nRadius--, ::GetSysColor(COLOR_3DLIGHT), ::GetSysColor(COLOR_3DSHADOW));
	DrawBitmap(pDC,s_ButStateNew);
	for ( j=0; j<m_nKnobNum; j++ ) {
		const CPoint ptKnobCenter = CalculateKnobPosi(m_nKnobPosi[j]);
		int nKnobRadius = m_nKnobRadius;	
		const CRect rcKnob(ptKnobCenter.x - nKnobRadius, ptKnobCenter.y - nKnobRadius, ptKnobCenter.x + nKnobRadius, ptKnobCenter.y + nKnobRadius);
		CRgn rgnKnob;
		rgnKnob.CreateEllipticRgnIndirect(rcKnob);
		CBrush brKnob(::GetSysColor(COLOR_BTNFACE));
		pDC->FillRgn(&rgnKnob, &brKnob);
		rgnKnob.DeleteObject();
		if(m_bDragging) {
			DrawCircle(pDC, ptKnobCenter, --nKnobRadius, ::GetSysColor(COLOR_3DDKSHADOW), ::GetSysColor(COLOR_3DHIGHLIGHT));
			DrawCircle(pDC, ptKnobCenter, --nKnobRadius, ::GetSysColor(COLOR_3DSHADOW), ::GetSysColor(COLOR_3DLIGHT));
		} else {
			DrawCircle(pDC, ptKnobCenter, --nKnobRadius, ::GetSysColor(COLOR_3DHIGHLIGHT), ::GetSysColor(COLOR_3DDKSHADOW));
			DrawCircle(pDC, ptKnobCenter, --nKnobRadius, ::GetSysColor(COLOR_3DLIGHT), ::GetSysColor(COLOR_3DSHADOW));
		}
		if(GetFocus() == this) {
			DrawCircle(pDC, ptKnobCenter, nKnobRadius-2, RGB(0, 0, 0), TRUE);
			if ( j == m_nKnobCrt ) {
				DrawCircle(pDC, ptKnobCenter, nKnobRadius-4, RGB(0, 0, 0), TRUE);
			}
		}
	}
	DrawText(pDC);
}

void KbutsliderCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default		
	if ( sqrt(pow((float)m_ptCenter.x - (float)point.x,2) + pow((float)m_ptCenter.y - (float)point.y,2)) < m_nRadius * 2 / 4 ) {
		s_ClikState = 0;
		RedrawWindow();
		OnLButtonUp(nFlags,s_ButStateNew);
	}
	if(m_bDragging) {
		m_bDragging = false;
		::ReleaseCapture();	
		PostMessageToParent(TB_THUMBTRACK);
		RedrawWindow();		
	} else {
		CSliderCtrl::OnLButtonUp(nFlags, point);
	}
}

void KbutsliderCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_nKnobCrt = DetectNearestKnob(GetCurentAngle(point));
	if ( sqrt(pow((float)m_ptCenter.x - (float)point.x,2) + pow((float)m_ptCenter.y - (float)point.y,2)) < m_nRadius * 2 / 4 ) {
		s_ClikState = 1;
		s_ButStateNew = 1 + s_ClikState;
		RedrawWindow();
		OnLButtonDown(nFlags,s_ButStateNew);
	}
	if(!m_bDragging) {
		m_bDragging = true;
		SetCapture();
		SetFocus();
		PostMessageToParent(TB_THUMBTRACK);
		RedrawWindow();
	} else {
		CSliderCtrl::OnLButtonDown(nFlags, point);
	}	
}

void KbutsliderCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	int angle;
	s_ButStateOld = s_ButStateNew;
	m_nKnobBfr = m_nKnobCrt;
	if ( sqrt(pow((float)m_ptCenter.x - (float)point.x,2) + pow((float)m_ptCenter.y - (float)point.y,2)) < m_nRadius * 2 / 4 ) {
		s_ButStateNew = 1 + s_ClikState;
	} else {
		s_ButStateNew = 0;
	}
	if ( sqrt(pow((float)m_ptCenter.x - (float)point.x,2) + pow((float)m_ptCenter.y - (float)point.y,2)) < m_nRadius * 3 / 4 ) {
		angle = GetCurentAngle(point);
		m_nKnobCrt = DetectNearestKnob(angle);		
	//	RedrawWindow();
	} else {
		if(m_bDragging) {
			m_nKnobPosi[m_nKnobCrt] = GetCurentAngle(point);
			PostMessageToParent(TB_THUMBTRACK);
			RedrawWindow();
		} else {
			CSliderCtrl::OnMouseMove(nFlags, point);
		}		
	}
	if ( (s_ButStateOld != s_ButStateNew) || (m_nKnobBfr != m_nKnobCrt) ) {
		RedrawWindow(); 
	}
}

/* ------------------------------------------------------------------------------------------ */

void KbutsliderCtrl::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	
	CSliderCtrl::OnTimer(nIDEvent);
}

void KbutsliderCtrl::Initialize () {
	CRect rc;
	GetClientRect(rc);
	// Resize the window to make it square
	rc.bottom = rc.right = min(rc.bottom, rc.right);
	// Get the vital statistics of the window
	m_ptCenter = rc.CenterPoint();
	m_nRadius = rc.bottom/2-(m_nKnobRadius+1);
	// Set the window region so mouse clicks only activate the round section of the slider
	m_rgn.DeleteObject(); 
	SetWindowRgn(NULL, FALSE);
	m_rgn.CreateEllipticRgnIndirect(rc);
	SetWindowRgn(m_rgn, TRUE);
//	VerifyPos();
	c_mybitmap.Attach(::LoadImage(::AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_BITMAP1), IMAGE_BITMAP,0,0,LR_LOADMAP3DCOLORS));
//	BITMAP	bitmapbits;
//	c_mybitmap.GetBitmap(&bitmapbits);
}

void KbutsliderCtrl::PostMessageToParent(const int nTBCode) const
{
	CWnd* pWnd = GetParent();
	if(pWnd) pWnd->PostMessage(WM_HSCROLL, (WPARAM)((GetPos() << 16) | nTBCode), (LPARAM)GetSafeHwnd());
}

/* ------------------------------------------------------------------------------------------ */

void KbutsliderCtrl::OnSize(UINT nType, int cx, int cy) {
	CSliderCtrl::OnSize(nType, cx, cy);
	Initialize();
	// TODO: Add your message handler code here
}

#pragma warning(disable:4100)
BOOL KbutsliderCtrl::OnEraseBkgnd(CDC* pDC)  {
	// TODO: Add your message handler code here and/or call default	
	return TRUE;
	// return CSliderCtrl::OnEraseBkgnd(pDC);
}
#pragma warning(default:4100)

BOOL KbutsliderCtrl::PreCreateWindow(CREATESTRUCT& cs)  {
	// TODO: Add your specialized code here and/or call the base class
	if(!CSliderCtrl::PreCreateWindow(cs)) return FALSE;
	SetRange(0, 359, FALSE);
	SetLineSize(1);
	SetPageSize(10);
	Initialize();
	return TRUE;
}

void KbutsliderCtrl::PreSubclassWindow() {
	// TODO: Add your specialized code here and/or call the base class
	CSliderCtrl::PreSubclassWindow();
	SetRange(0, 359, FALSE);
	SetLineSize(1);
	SetPageSize(10);
	Initialize();
}

void KbutsliderCtrl::DoDataExchange(CDataExchange* pDX)  {
	// TODO: Add your specialized code here and/or call the base class
	CSliderCtrl::DoDataExchange(pDX);
}

/* ------------------------------------------------------------------------------------------ */

void KbutsliderCtrl::SetKnobsNum(int no) {
	m_nKnobNum = no % 16;
	if ( this->IsWindowVisible() ) RedrawWindow();
}

void KbutsliderCtrl::SetKnobPosi(int kn, int pos) {
	m_nKnobPosi[kn % 16] = pos % 360;
	if ( this->IsWindowVisible() ) RedrawWindow();
}

int KbutsliderCtrl::GetKnobPosi(int kn) {
	return m_nKnobPosi[kn % 16];
}

void KbutsliderCtrl::OnLButtonDown(UINT nFlags, int kns)
{
//	MessageBox( "me :");
}

void KbutsliderCtrl::OnLButtonUp(UINT nFlags, int kns)
{
//	MessageBox( "you :");
}
