//////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/ForceFeedback/BasicImpl/DiscoveryCallbackImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * RTTC::ForceFeedback::BasicImpl::DiscoveryCallbackImpl.
 *
 * This file contains the definition of the methods of the
 * RTTC::ForceFeedback::BasicImpl::DiscoveryCallbackImpl class.
 */

#include "DiscoveryCallbackImpl.h"

#include <string>
#include <exception>
#include <iostream>

///////////////////////////////////////////////////////////////////////////////
RTTC::ForceFeedback::BasicImpl::
DiscoveryCallbackImpl::
~DiscoveryCallbackImpl()
{
  // Basic Implementation has no required behavior in callback
  // destructor. Might need to free resources, close database connections, etc.
}

///////////////////////////////////////////////////////////////////////////////
RTTC::ForceFeedback::BasicImpl::
DiscoveryCallbackImpl::
DiscoveryCallbackImpl(
  RTTC::ForceFeedback::ProxyPtr const & pProxy,
  CallbackInfoPtr const & pCallbackInfo
)
  :
  _pCallbackInfo( pCallbackInfo ),
  _pProxy( pProxy )
{
  // Basic DiscoveryCallback implementation requires no behavior at callback
  // creation. This is fairly typical -- most callback constructor will not
  // have behavior
}


///////////////////////////////////////////////////////////////////////////////
void
RTTC::ForceFeedback::BasicImpl::
DiscoveryCallbackImpl::
execute()
{
  // Discovered proxy objects must be maintained by the application to
  // receive state change and destruction callbacks for that object. In this
  // example basic implementation, the proxy object is maintained within a
  // list of discovered objects.
  _pCallbackInfo->getDiscoveredSDOlist().push_back( _pProxy );

  if ( 1 < _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << "RTTC::ForceFeedback::BasicImpl DiscoveryCallbackImpl execute()"
       << std::endl;
    os << "SDO ID " << _pProxy->getSDOid() << std::endl;
    os << "Version " << _pProxy->getPublicationState()->get_RTTC_ForceFeedback_VersionCounter() << std::endl;
  }
  else if ( 1 == _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << 'd' << std::flush;
  }

  // Callbacks should only catch exceptions if they can actually HANDLE the
  // exception -- i.e. correct the error condition.  Otherwise, the exception
  // will propagate up to the evokeCallbacks invocation where it may be handled
  // in an application specific manner.
}
