/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/TASemulator/BasicImpl/StateChangeCallbackImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * RTTC::TASemulator::BasicImpl::StateChangeCallbackImpl.
 *
 * This file contains the definition of the methods of the
 * RTTC::TASemulator::BasicImpl::StateChangeCallbackImpl class.
 */

#include "StateChangeCallbackImpl.h"


// For SDO and any contained SDO types, include headers to allow access to
// SDO publication state.
#include <RTTC/TASemulator/ImmutablePublicationState.h>
#include <RTTC/TASemulator/Proxy.h>

// For any localclass or message types, include the header and factory
// *implementation* needed to create and access the local class
#include <TENA/UniqueID/Includes.h>


#include <string>
#include <exception>
#include <iostream>
#include <iomanip>

///////////////////////////////////////////////////////////////////////////////
RTTC::TASemulator::BasicImpl::
StateChangeCallbackImpl::
~StateChangeCallbackImpl()
{
  // Basic Implementation has no required behavior in state change callback
  // destructor.  Might need to free resources, close database connections,
  // etc.
}

///////////////////////////////////////////////////////////////////////////////
RTTC::TASemulator::BasicImpl::
StateChangeCallbackImpl::
StateChangeCallbackImpl(
  RTTC::TASemulator::ProxyPtr const & pProxy,
  CallbackInfoPtr const & pCallbackInfo
)
  :
  _pCallbackInfo( pCallbackInfo ),
  _pProxy( pProxy )
{
  // Basic StateChangeCallback implementation requires no behavior at callback
  // creation.  This is fairly typical--most callback constructors will not
  // have behavior.
}

///////////////////////////////////////////////////////////////////////////////
void
RTTC::TASemulator::BasicImpl::
StateChangeCallbackImpl::
execute()
{
  if ( 1 < _pCallbackInfo->getVerbosity() )
  {
    // This is just an example that performs some "update" every time the state
    // changes; in this case, printing out the new SDO state.
    std::ostream & os = _pCallbackInfo->getOutputStream();

    os << "RTTC::TASemulator::BasicImpl StateChangeCallbackImpl execute()"
       << std::endl;
    os << "SDO ID " << _pProxy->getSDOid() << std::endl;
    os << "Version "
       << _pProxy->getPublicationState()->get_RTTC_TASemulator_VersionCounter()
       << std::endl;
  }
  else if ( 1 == _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << 'c' << std::flush;
  }

  if ( 2 < _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    //
    // Get the publicationState of the
    // RTTC::TASemulator SDO '_pProxy'.
    //
    RTTC::TASemulator::ImmutablePublicationStatePtr
      this_TASemulator_PubState( _pProxy->getPublicationState() );
    os << std::setw(0) << " " << "this.RTTC_TASemulator_VersionCounter "
              << this_TASemulator_PubState->get_RTTC_TASemulator_VersionCounter() << std::endl;
    os << std::setw(0) << " " << "this.fieldOfView ";
    os << this_TASemulator_PubState->get_fieldOfView() << std::endl;
    os << std::setw(0) << " " << "this.zoomFactor ";
    os << this_TASemulator_PubState->get_zoomFactor() << std::endl;
    os << std::setw(0) << " " << "this.terrainID ";
    os << this_TASemulator_PubState->get_terrainID() << std::endl;
    os << std::setw(0) << " " << "this.viewFromX ";
    os << this_TASemulator_PubState->get_viewFromX() << std::endl;
    os << std::setw(0) << " " << "this.viewFromY ";
    os << this_TASemulator_PubState->get_viewFromY() << std::endl;
    os << std::setw(0) << " " << "this.viewFromZ ";
    os << this_TASemulator_PubState->get_viewFromZ() << std::endl;
    os << std::setw(0) << " " << "this.viewToX ";
    os << this_TASemulator_PubState->get_viewToX() << std::endl;
    os << std::setw(0) << " " << "this.viewToY ";
    os << this_TASemulator_PubState->get_viewToY() << std::endl;
    os << std::setw(0) << " " << "this.viewToZ ";
    os << this_TASemulator_PubState->get_viewToZ() << std::endl;
    os << std::setw(0) << " " << "this.viewPsi ";
    os << this_TASemulator_PubState->get_viewPsi() << std::endl;
    os << std::setw(0) << " " << "this.viewTheta ";
    os << this_TASemulator_PubState->get_viewTheta() << std::endl;
    os << std::setw(0) << " " << "this.viewPhi ";
    os << this_TASemulator_PubState->get_viewPhi() << std::endl;
    os << std::setw(0) << " " << "this.TASemulatorID ";
    TENA::UniqueID::Pointer TASemulatorID_LocalClass_Ptr(
      this_TASemulator_PubState->get_TASemulatorID() );

    os << std::endl;
    os << std::setw(4) << " " << "TASemulatorID.siteID ";
    os << TASemulatorID_LocalClass_Ptr->get_siteID() << std::endl;
    os << std::setw(4) << " " << "TASemulatorID.applicationID ";
    os << TASemulatorID_LocalClass_Ptr->get_applicationID() << std::endl;
    os << std::setw(4) << " " << "TASemulatorID.objectID ";
    os << TASemulatorID_LocalClass_Ptr->get_objectID() << std::endl;
  }

  // Callbacks should only catch exceptions if they can actually HANDLE the
  // exception--i.e. correct the error condition.  Otherwise, the exception
  // will propagate up to the evokeCallbacks invocation where it may be handled
  // in an application specific manner.
}
