/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/Platform/BasicImpl/DiscoveryCallbackImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * TENA::Platform::BasicImpl::DiscoveryCallbackImpl.
 *
 * This file contains the declaration of and documentation for the
 * TENA::Platform::BasicImpl::DiscoveryCallbackImpl class.
 */

#ifndef TENA_Platform_BasicImpl_DiscoveryCallbackImpl_h
#define TENA_Platform_BasicImpl_DiscoveryCallbackImpl_h

#include <DCT/System/config.h>
#include <basicImpl_TENA_Platform_v3_1_EXPORT.h>
#include <TENA/Middleware/Callback/Callback.h>
#include <TENA/Platform/Proxy.h>
#include "CallbackInfo.h"

namespace TENA
{
  namespace Platform
  {
    namespace BasicImpl
    {
      /*!\class DiscoveryCallbackImpl DiscoveryCallbackImpl.h \
         TENA/Platform/BasicImpl/DiscoveryCallbackImpl.h
       *
       * \brief Callback created in response to discovery of an SDO
       *
       * When an SDO is discovered, a DiscoveryCallback is created.  This
       * basic implementation simply adds the discovered SDO to list of SDOs.
       * To alter the behavior, modify the execute method of the callback.
       */
      class basicImpl_TENA_Platform_v3_1_EXPORT DiscoveryCallbackImpl
        : public virtual TENA::Middleware::Callback::Callback
      {
      public:
        //! This is the only constructor.
        /*!
          \param[in] pProxy ProxyPtr for the Servant being discovered
          \param[in] pCallbackInfo CallbackInfo containing the list of discovered ProxyPtrs

          This basic implementation simply adds the discovered SDO ProxyPtr
          to the list of discovered SDOs in the CallbackInfo.  It is necessary
          for something in in the application to hold a reference to
          discovered SDO proxies or they will be unsubscribed and released.
          The list in the CallbackInfo accomplishes this.
        */
        DiscoveryCallbackImpl(
          TENA::Platform::ProxyPtr const & pProxy,
          CallbackInfoPtr const & pCallbackInfo );

        //! Instances are destructed after invoking the execute() method.
        ~DiscoveryCallbackImpl();

        //! Perform the behavior associated with this callback.
        /*!
         * Basic implementation behavior just adds the discovered SDO ProxyPtr
         * to the std::list in the CallbackInfo
         */
        void
        execute();

      private:
        //! Default copy construction is disallowed.
        DiscoveryCallbackImpl(
          DiscoveryCallbackImpl const & rhs );
        //! Default copy assignment is disallowed.
        DiscoveryCallbackImpl & operator=(
          DiscoveryCallbackImpl const & rhs );

        //! Hold a CallbackInfo
        CallbackInfoPtr  _pCallbackInfo;

        //! Hold a reference to the discovered SDO ProxyPtr
        TENA::Platform::ProxyPtr _pProxy;
      };
#ifndef DOXYGEN_SHOULD_SKIP_THIS
      // Typedef to prevent eclipsing of this class by function of the same name
      typedef DiscoveryCallbackImpl DiscoveryCallbackImpl;
#endif // DOXYGEN_SHOULD_SKIP_THIS
    } // End of namespace BasicImpl
  } // End of namespace Platform
} // End of namespace TENA

#endif // ifndef TENA_Platform_BasicImpl_DiscoveryCallbackImpl_h
