/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/Platform/BasicImpl/StateChangeCallbackImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * TENA::Platform::BasicImpl::StateChangeCallbackImpl.
 *
 * This file contains the definition of the methods of the
 * TENA::Platform::BasicImpl::StateChangeCallbackImpl class.
 */

#include "StateChangeCallbackImpl.h"


// For SDO and any contained SDO types, include headers to allow access to
// SDO publication state.
#include <TENA/Platform/ImmutablePublicationState.h>
#include <TENA/Platform/Proxy.h>

// For any localclass or message types, include the header and factory
// *implementation* needed to create and access the local class
#include <TENA/UniqueID/Includes.h>
#include <TENA/Time/Includes.h>
#include <TENA/CoordinatedUniversalTime/Includes.h>
#include <TENA/Date/Includes.h>
#include <TENA/GpsTime/Includes.h>
#include <TENA/UnixTime/Includes.h>
#include <TENA/PlatformType/Includes.h>
#include <DIS/EntityType/Includes.h>
#include <TENA/TSPI/Includes.h>
#include <TENA/AngularAcceleration/Includes.h>
#include <TENA/AngularVelocity/Includes.h>
#include <TENA/GeocentricAcceleration/Includes.h>
#include <TENA/GeocentricSpatialReferenceFrame/Includes.h>
#include <TENA/SpatialReferenceFrame/Includes.h>
#include <TENA/GeocentricOrientationEuler/Includes.h>
#include <TENA/GeocentricPosition/Includes.h>
#include <TENA/GeocentricVelocity/Includes.h>
#include <TENA/GeodeticAcceleration/Includes.h>
#include <TENA/GeodeticSpatialReferenceFrame/Includes.h>
#include <TENA/GeodeticOrientationEuler/Includes.h>
#include <TENA/GeodeticPosition/Includes.h>
#include <TENA/GeodeticVelocity/Includes.h>
#include <TENA/LocalSphericalTangentPlaneAcceleration/Includes.h>
#include <TENA/LocalSphericalTangentPlaneSpatialReferenceFrame/Includes.h>
#include <TENA/LocalSphericalTangentPlaneOrientationEuler/Includes.h>
#include <TENA/LocalSphericalTangentPlanePosition/Includes.h>
#include <TENA/LocalSphericalTangentPlaneVelocity/Includes.h>
#include <TENA/LocalTangentPlaneENUacceleration/Includes.h>
#include <TENA/LocalTangentPlaneENUspatialReferenceFrame/Includes.h>
#include <TENA/LocalTangentPlaneENUorientationEuler/Includes.h>
#include <TENA/LocalTangentPlaneENUposition/Includes.h>
#include <TENA/LocalTangentPlaneENUvelocity/Includes.h>
#include <TENA/Acceleration/Includes.h>
#include <TENA/Orientation/Includes.h>
#include <TENA/Position/Includes.h>
#include <TENA/Velocity/Includes.h>


#include <string>
#include <exception>
#include <iostream>
#include <iomanip>

///////////////////////////////////////////////////////////////////////////////
TENA::Platform::BasicImpl::
StateChangeCallbackImpl::
~StateChangeCallbackImpl()
{
  // Basic Implementation has no required behavior in state change callback
  // destructor.  Might need to free resources, close database connections,
  // etc.
}

///////////////////////////////////////////////////////////////////////////////
TENA::Platform::BasicImpl::
StateChangeCallbackImpl::
StateChangeCallbackImpl(
  TENA::Platform::ProxyPtr const & pProxy,
  CallbackInfoPtr const & pCallbackInfo
)
  :
  _pCallbackInfo( pCallbackInfo ),
  _pProxy( pProxy )
{
  // Basic StateChangeCallback implementation requires no behavior at callback
  // creation.  This is fairly typical--most callback constructors will not
  // have behavior.
}

///////////////////////////////////////////////////////////////////////////////
void
TENA::Platform::BasicImpl::
StateChangeCallbackImpl::
execute()
{
  if ( 1 < _pCallbackInfo->getVerbosity() )
  {
    // This is just an example that performs some "update" every time the state
    // changes; in this case, printing out the new SDO state.
    std::ostream & os = _pCallbackInfo->getOutputStream();

/*    os << "TENA::Platform::BasicImpl StateChangeCallbackImpl execute()"
       << std::endl;
    os << "SDO ID " << _pProxy->getSDOid() << std::endl;
    os << "Version "
       << _pProxy->getPublicationState()->get_TENA_Platform_VersionCounter()
       << std::endl;*/
  }
  else if ( 1 == _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
   //mg os << 'c' << std::flush;
  }

  if ( 2 < _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    //
    // Get the publicationState of the
    // TENA::Platform SDO '_pProxy'.
    //
    TENA::Platform::ImmutablePublicationStatePtr
      this_Platform_PubState( _pProxy->getPublicationState() );
    os << std::setw(0) << " " << "this.TENA_Platform_VersionCounter "
              << this_Platform_PubState->get_TENA_Platform_VersionCounter() << std::endl;
    os << std::setw(0) << " " << "this.platformID ";
    TENA::UniqueID::Pointer platformID_LocalClass_Ptr(
      this_Platform_PubState->get_platformID() );

    os << std::endl;
    os << std::setw(4) << " " << "platformID.siteID ";
    os << platformID_LocalClass_Ptr->get_siteID() << std::endl;
    os << std::setw(4) << " " << "platformID.applicationID ";
    os << platformID_LocalClass_Ptr->get_applicationID() << std::endl;
    os << std::setw(4) << " " << "platformID.objectID ";
    os << platformID_LocalClass_Ptr->get_objectID() << std::endl;
    os << std::setw(0) << " " << "this.timeStamp ";
    TENA::Time::Pointer timeStamp_LocalClass_Ptr(
      this_Platform_PubState->get_timeStamp() );

    os << std::endl;
    os << std::setw(0) << " " << "this.callSign ";
    os << this_Platform_PubState->get_callSign() << std::endl;
    os << std::setw(0) << " " << "this.designator ";
    os << this_Platform_PubState->get_designator() << std::endl;
    os << std::setw(0) << " " << "this.platformType ";
    TENA::PlatformType::Pointer platformType_LocalClass_Ptr(
      this_Platform_PubState->get_platformType() );

    os << std::endl;
    os << std::setw(4) << " " << "platformType.typeIndex ";
    os << platformType_LocalClass_Ptr->get_typeIndex() << std::endl;
    os << std::setw(4) << " " << "platformType.databaseVersion ";
    os << platformType_LocalClass_Ptr->get_databaseVersion() << std::endl;
    os << std::setw(0) << " " << "this.affiliation ";
    os << this_Platform_PubState->get_affiliation() << std::endl;
    os << std::setw(0) << " " << "this.damageState ";
    os << this_Platform_PubState->get_damageState() << std::endl;
    os << std::setw(0) << " " << "this.damagePercent ";
    os << this_Platform_PubState->get_damagePercent() << std::endl;
    os << std::setw(0) << " " << "this.tspi ";
    TENA::TSPI::Pointer tspi_LocalClass_Ptr(
      this_Platform_PubState->get_tspi() );

//	tspi_LocalClass_Ptr->get_LocalTangentPlaneENUposition(_pLTPSRF)->get_x();

    os << std::endl;
  }

  // Callbacks should only catch exceptions if they can actually HANDLE the
  // exception--i.e. correct the error condition.  Otherwise, the exception
  // will propagate up to the evokeCallbacks invocation where it may be handled
  // in an application specific manner.
}
