/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/Platform/BasicImpl/SubscriptionInfoImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * TENA::Platform::BasicImpl::SubscriptionInfoImpl.
 *
 * This file contains the declaration of and documentation for the
 * TENA::Platform::BasicImpl::SubscriptionInfoImpl class.
 */

#ifndef TENA_Platform_BasicImpl_SubscriptionInfoImpl_h
#define TENA_Platform_BasicImpl_SubscriptionInfoImpl_h

#include <DCT/System/config.h>
#include <basicImpl_TENA_Platform_v3_1_EXPORT.h>
#include "CallbackInfo.h"
#include <TENA/Platform/SubscriptionInfo.h>

namespace TENA
{
  namespace Platform
  {
    namespace BasicImpl
    {
      /*!\class SubscriptionInfoImpl SubscriptionInfoImpl.h \
         TENA/Platform/BasicImpl/SubscriptionInfoImpl.h
       *
       * \brief Holder for all user-defined objects required to subscribe
       * to this SDO type.
       *
       * The basic implementation for this object should be sufficient
       * for most applications.
       *
       * Any custom variables needed by the callback factories can be
       * passed in through the CallbackInfo object.
       */
      class basicImpl_TENA_Platform_v3_1_EXPORT SubscriptionInfoImpl
        : public TENA::Platform::SubscriptionInfo
      {
      public:
        // If no callbacks are desired, construct a default SubscriptionInfo
        // from the OM definition.  This is typically used to dereference an
        // SDO pointer.

        /*!
         * This ctor creates callback factories using the provided
         * CallbackInfoPtr .
         *
         * Use this ctor when subscribing to this SDO type.
         */
        explicit SubscriptionInfoImpl( CallbackInfoPtr const & );

        virtual
        ~SubscriptionInfoImpl();

      private:
        //! Default copy construction is disallowed.
        SubscriptionInfoImpl(
          SubscriptionInfoImpl const & rhs );
        //! Default copy assignment is disallowed.
        SubscriptionInfoImpl & operator=(
          SubscriptionInfoImpl const & rhs );

      };
    } // End of namespace BasicImpl
  } // End of namespace Platform
} // End of namespace TENA

#endif // ifndef TENA_Platform_BasicImpl_SubscriptionInfoImpl_h
