//////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/PlatformExtension/BasicImpl/StateChangeCallbackImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * TENA::PlatformExtension::BasicImpl::StateChangeCallbackImpl.
 *
 * This file contains the declaration of and documentation for the
 * TENA::PlatformExtension::BasicImpl::StateChangeCallbackImpl class.
 */

#ifndef TENA_PlatformExtension_BasicImpl_StateChangeCallbackImpl_h
#define TENA_PlatformExtension_BasicImpl_StateChangeCallbackImpl_h

#include <DCT/System/config.h>
#include <basicImpl_TENA_Platform_v3_1_EXPORT.h>
#include <TENA/Middleware/Callback/Callback.h>
#include <TENA/PlatformExtension/Proxy.h>

#include "CallbackInfo.h"

namespace TENA
{
  namespace PlatformExtension
  {
    namespace BasicImpl
    {
      /*!\class StateChangeCallbackImpl StateChangeCallbackImpl.h \
         TENA/PlatformExtension/BasicImpl/StateChangeCallbackImpl.h
       *
       * \brief Callback created in response to state change of an SDO
       *
       * When an SDO state changes, a StateChangeCallback is created.  This
       * basic simply writes the changes state to an output stream.
       * To modify the behavior, modify the execute method of the callback (and possibly the
       * state information in the CallbackInfo object.
       */
      class basicImpl_TENA_Platform_v3_1_EXPORT StateChangeCallbackImpl
        : public virtual TENA::Middleware::Callback::Callback
      {
      public:
        //! This is the only constructor.
        /*!
          \param[in] pProxy The ProxyPtr for the Servant whose state has changed
          \param[in] pCallbackInfo The CallbackInfo containing the list of ProxyPtrs that have been discovered

          This basic implementation simply uses the OutputStream in the CallbackInfo
          to print the updated state
        */
        StateChangeCallbackImpl(
          TENA::PlatformExtension::ProxyPtr const & pProxy,
          CallbackInfoPtr const & pCallbackInfo );

        //! Instances are destructed after invoking the execute() method.
        ~StateChangeCallbackImpl();

        //! Perform the behavior associated with this callback.
        /*!
         * Basic implementation behavior just adds the discovered SDO ProxyPtr
         * to the std::list in the CallbackInfo
         */
        void
        execute();

      private:
        //! Default copy construction is disallowed.
        StateChangeCallbackImpl(
          StateChangeCallbackImpl const & rhs );
        //! Default copy assignment is disallowed.
        StateChangeCallbackImpl & operator=(
          StateChangeCallbackImpl const & rhs );

        //! Hold a CallbackInfo
        TENA::PlatformExtension::BasicImpl::CallbackInfoPtr _pCallbackInfo;

        //! Hold a reference to the SDO ProxyPtr whose state is changing
        TENA::PlatformExtension::ProxyPtr _pProxy;
      };
    } // End of namespace BasicImpl
  } // End of namespace PlatformExtension
} // End of namespace TENA


#endif // ifndef TENA_PlatformExtension_BasicImpl_StateChangeCallbackImpl_h
